@@ -7,7 +7,7 @@
 
 .jinit()
 jv <- .jcall("java/lang/System", "S", "getProperty", "java.runtime.version")
-if(substr(jv, 1L, 1L) == "1") {
+if(substr(jv, 1L, 2L) == "1.") {
   jvn <- as.numeric(paste0(strsplit(jv, "[.]")[[1L]][1:2], collapse = "."))
   if(jvn < 1.5) stop("Java 5 or higher is needed for this package but not available")
 }
