@@ -1,14 +1,7 @@
 import java.util.ArrayList;
 
-import moa.cluster.Cluster;
-import moa.cluster.Clustering;
 import moa.clusterers.AbstractClusterer;
-import moa.clusterers.macro.dbscan.DBScan;
-import moa.core.Measurement;
-import moa.options.FloatOption;
-import moa.options.IntOption;
 import weka.core.DenseInstance;
-import weka.core.Instance;
 
 class StreamMOA {
   public static void update(AbstractClusterer clusterer, double[][] points) {
@@ -19,4 +12,4 @@ class StreamMOA {
         clusterer.trainOnInstanceImpl(p);
     }
   }
-}
\ No newline at end of file
+}
